/*
 *  This file is part of libosc++.
 *
 *  libosc++ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  libosc++ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesse General Public License
 *  along with libosc++.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright 2009 Cristian Maglie
 *
 *  Cristian Maglie <liboscpp@bug.st>
 *  http://bug.st/libosc++
 */

#include <libosc++/bundle.h>
#include <endian_utils.h>

namespace osc {

Bundle::Bundle() :
	started(false), closed(false), data("")
{
	// Empty
}

Bundle::~Bundle() 
{
	// Empty
}

void Bundle::clear()
{
	data = "";
	started = false;
	closed = false;
}

void Bundle::begin(const uint64_t _time)
{
	if (started || closed)
		throw BundleAlreadyOpenedException();
	data += "#bundle";
	data += '\0';
	appendInt64(data, _time);
	started = true;
}

void Bundle::writeMessage(Message &_msg)
{
	if (!started)
		throw BundleNotOpenedExecption();
	appendInt32(data, _msg.size());
	_msg.output(data);
}

void Bundle::writeBundle(Bundle &_bnd)
{
	if (!started)
		throw BundleNotOpenedExecption();
	appendInt32(data, _bnd.size());
	_bnd.output(data);
}

void Bundle::writeRaw(const string &_data)
{
	if (!started)
		throw BundleNotOpenedExecption();
	appendInt32(data, _data.size());
	data.append(_data);
}

void Bundle::end()
{
	if (!started)
		throw BundleNotOpenedExecption();
	started = false;
	closed = true;
}

int Bundle::size()
{
	if (!closed)
		throw BundleNotClosedException();
	return data.size();
}

void Bundle::output(char *_buffer)
{
	if (!closed)
		throw BundleNotClosedException();
	data.copy(_buffer, data.size());
}

void Bundle::output(string &_buffer)
{
	if (!closed)
		throw BundleNotClosedException();
	_buffer.append(data);
}

} // namespace osc
