/*
 *  This file is part of libosc++.
 *
 *  libosc++ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  libosc++ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesse General Public License
 *  along with libosc++.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright 2009 Cristian Maglie
 *
 *  Cristian Maglie <liboscpp@bug.st>
 *  http://bug.st/libosc++
 */

#ifndef ENDIAN_UTILS_H_INCLUDED
#define ENDIAN_UTILS_H_INCLUDED

#include <config.h>

static inline int32_t convInt32(int32_t _x)
{
#ifdef WORDS_BIGENDIAN
	return _x;
#else
	union{
		int32_t i;
		char c[4];
	} u;
	char t;
	u.i = _x;
	t = u.c[3];
	u.c[3] = u.c[0];
	u.c[0] = t;
	t = u.c[2];
	u.c[2] = u.c[1];
	u.c[1] = t;
	return u.i;
#endif
}

static inline void appendInt32(string &_buff, int32_t _v) 
{
	_v = convInt32(_v);
	char *p = reinterpret_cast<char *>(&_v);
	_buff.append(p, 4);
}

static inline int32_t readInt32(string &_buff, int &_pos)
{
	int32_t r;
	char *rp = reinterpret_cast<char *>(&r);
#ifdef WORDS_BIGENDIAN
	rp[0] = _buff[_pos++];
	rp[1] = _buff[_pos++];
	rp[2] = _buff[_pos++];
	rp[3] = _buff[_pos++];
#else
	rp[3] = _buff[_pos++];
	rp[2] = _buff[_pos++];
	rp[1] = _buff[_pos++];
	rp[0] = _buff[_pos++];
#endif
	return r;
}

static inline uint32_t convUInt32(uint32_t _x)
{
#ifdef WORDS_BIGENDIAN
	return _x;
#else
	union{
		uint32_t i;
		char c[4];
	} u;
	char t;
	u.i = _x;
	t = u.c[3];
	u.c[3] = u.c[0];
	u.c[0] = t;
	t = u.c[2];
	u.c[2] = u.c[1];
	u.c[1] = t;
	return u.i;
#endif
}

static inline int64_t convInt64(int64_t _x)
{
#ifdef WORDS_BIGENDIAN
	return _x;
#else
	union{
		int64_t i;
		char c[8];
	} u;
	char t;
	u.i = _x;
	t = u.c[7];
	u.c[7] = u.c[0];
	u.c[0] = t;
	t = u.c[6];
	u.c[6] = u.c[1];
	u.c[1] = t;
	t = u.c[5];
	u.c[5] = u.c[2];
	u.c[2] = t;
	t = u.c[4];
	u.c[4] = u.c[3];
	u.c[3] = t;
	return u.i;
#endif
}

static inline void appendInt64(string &_buff, int64_t _v) 
{
	_v = convInt64(_v);
	char *p = reinterpret_cast<char *>(&_v);
	_buff.append(p, 8);
}

static inline int64_t readInt64(string &_buff, int &_pos)
{
	int64_t r;
	char *rp = reinterpret_cast<char *>(&r);
#ifdef WORDS_BIGENDIAN
	rp[0] = _buff[_pos++];
	rp[1] = _buff[_pos++];
	rp[2] = _buff[_pos++];
	rp[3] = _buff[_pos++];
	rp[4] = _buff[_pos++];
	rp[5] = _buff[_pos++];
	rp[6] = _buff[_pos++];
	rp[7] = _buff[_pos++];
#else
	rp[7] = _buff[_pos++];
	rp[6] = _buff[_pos++];
	rp[5] = _buff[_pos++];
	rp[4] = _buff[_pos++];
	rp[3] = _buff[_pos++];
	rp[2] = _buff[_pos++];
	rp[1] = _buff[_pos++];
	rp[0] = _buff[_pos++];
#endif
	return r;
}

static inline void convUInt64(char *p, uint64_t x)
{
#ifdef WORDS_BIGENDIAN
	*reinterpret_cast<uint64_t *>(p) = x;
#else
	union{
		uint64_t i;
		char c[8];
	} u;
	u.i = x;
	p[7] = u.c[0];
	p[6] = u.c[1];
	p[5] = u.c[2];
	p[4] = u.c[3];
	p[3] = u.c[4];
	p[2] = u.c[5];
	p[1] = u.c[6];
	p[0] = u.c[7];
#endif
}

static inline void align32(string &_v)
{
	while ((_v.size() % 4) != 0)
		_v += '\0';
}

static inline void appendFloat32(string &_buff, float _v)
{
#ifdef WORDS_BIGENDIAN
	char *p = reinterpret_cast<char *>(&_v);
	_buff.append(p, 8);
#else
	char *p = reinterpret_cast<char *>(&_v) + 3;
	_buff += *(p--);
	_buff += *(p--);
	_buff += *(p--);
	_buff += *p;
#endif
}

static inline float readFloat32(string &_buff, int &_pos)
{
	float r;
	char *rp = reinterpret_cast<char *>(&r);
#ifdef WORDS_BIGENDIAN
	rp[0] = _buff[_pos++];
	rp[1] = _buff[_pos++];
	rp[2] = _buff[_pos++];
	rp[3] = _buff[_pos++];
#else
	rp[3] = _buff[_pos++];
	rp[2] = _buff[_pos++];
	rp[1] = _buff[_pos++];
	rp[0] = _buff[_pos++];
#endif
	return r;
}

static inline void appendFloat64(string &_buff, double _v)
{
#ifdef WORDS_BIGENDIAN
	char *p = reinterpret_cast<char *>(&_v);
	_buff.append(p, 8);
#else
	char *p = reinterpret_cast<char *>(&_v) + 7;
	_buff += *(p--);
	_buff += *(p--);
	_buff += *(p--);
	_buff += *(p--);
	_buff += *(p--);
	_buff += *(p--);
	_buff += *(p--);
	_buff += *p;
#endif
}

static inline double readFloat64(string &_buff, int &_pos)
{
	double r;
	char *rp = reinterpret_cast<char *>(&r);
#ifdef WORDS_BIGENDIAN
	rp[0] = _buff[_pos++];
	rp[1] = _buff[_pos++];
	rp[2] = _buff[_pos++];
	rp[3] = _buff[_pos++];
	rp[4] = _buff[_pos++];
	rp[5] = _buff[_pos++];
	rp[6] = _buff[_pos++];
	rp[7] = _buff[_pos++];
#else
	rp[7] = _buff[_pos++];
	rp[6] = _buff[_pos++];
	rp[5] = _buff[_pos++];
	rp[4] = _buff[_pos++];
	rp[3] = _buff[_pos++];
	rp[2] = _buff[_pos++];
	rp[1] = _buff[_pos++];
	rp[0] = _buff[_pos++];
#endif
	return r;
}

#endif // ENDIAN_UTILS_H_INCLUDED

