/*
 *  This file is part of libosc++.
 *
 *  libosc++ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  libosc++ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesse General Public License
 *  along with libosc++.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright 2009 Cristian Maglie
 *
 *  Cristian Maglie <liboscpp@bug.st>
 *  http://bug.st/libosc++
 */

#ifndef DISPATCHER_H_INDCLUDED
#define DISPATCHER_H_INDCLUDED

#include <libosc++/exception.h>
#include <libosc++/message.h>

#include <string>
#include <iostream>

using namespace std;

namespace osc {

inline void chain(MessagePtr _m, const string &_addr, void (*_cb)())
{
	if (_addr != _m->getAddress())
		return;
	if (_m->getTag().length())
		return;
	_cb();
}

template<typename T1>
void chain(MessagePtr _m, const string &_addr, void (*_cb)(T1))
{
	if (_addr != _m->getAddress())
		return;
	char tag[2];
	tag[0] = oscType<T1>::tag;
	tag[1] = '\0';
	if (_m->getTag() != tag)
		return;
	T1 p1; _m->get(p1);
	_cb(p1);
}

template<typename T1, typename T2>
void chain(MessagePtr _m, const string &_addr, void (*_cb)(T1, T2))
{
	if (_addr != _m->getAddress())
		return;
	char tag[3];
	tag[0] = oscType<T1>::tag;
	tag[1] = oscType<T2>::tag;
	tag[2] = '\0';
	if (_m->getTag() != tag)
		return;
	T1 p1; _m->get(p1);
	T2 p2; _m->get(p2);
	_cb(p1, p2);
}

template<typename T1, typename T2, typename T3>
void chain(MessagePtr _m, const string &_addr, void (*_cb)(T1, T2, T3))
{
	if (_addr != _m->getAddress())
		return;
	char tag[4];
	tag[0] = oscType<T1>::tag;
	tag[1] = oscType<T2>::tag;
	tag[2] = oscType<T3>::tag;
	tag[3] = '\0';
	if (_m->getTag() != tag)
		return;
	T1 p1; _m->get(p1);
	T2 p2; _m->get(p2);
	T3 p3; _m->get(p3);
	_cb(p1, p2, p3);
}

template<typename T1, typename T2, typename T3, typename T4>
void chain(MessagePtr _m, const string &_addr, void (*_cb)(T1, T2, T3, T4))
{
	if (_addr != _m->getAddress())
		return;
	char tag[5];
	tag[0] = oscType<T1>::tag;
	tag[1] = oscType<T2>::tag;
	tag[2] = oscType<T3>::tag;
	tag[3] = oscType<T4>::tag;
	tag[4] = '\0';
	if (_m->getTag() != tag)
		return;
	T1 p1; _m->get(p1);
	T2 p2; _m->get(p2);
	T3 p3; _m->get(p3);
	T4 p4; _m->get(p4);
	_cb(p1, p2, p3, p4);
}

template<typename T1, typename T2, typename T3, typename T4, typename T5>
void chain(MessagePtr _m, const string &_addr, void (*_cb)(T1, T2, T3, T4, T5))
{
	if (_addr != _m->getAddress())
		return;
	char tag[6];
	tag[0] = oscType<T1>::tag;
	tag[1] = oscType<T2>::tag;
	tag[2] = oscType<T3>::tag;
	tag[3] = oscType<T4>::tag;
	tag[4] = oscType<T5>::tag;
	tag[5] = '\0';
	if (_m->getTag() != tag)
		return;
	T1 p1; _m->get(p1);
	T2 p2; _m->get(p2);
	T3 p3; _m->get(p3);
	T4 p4; _m->get(p4);
	T5 p5; _m->get(p5);
	_cb(p1, p2, p3, p4, p5);
}

template<typename OBJ>
void chain(MessagePtr _m, const string &_addr, OBJ *_obj, void (OBJ::*_cb)())
{
	if (_addr != _m->getAddress())
		return;
	if (_m->getTag().length())
		return;
	(_obj->*_cb)();
}

template<typename OBJ, typename T1>
void chain(MessagePtr _m, const string &_addr, OBJ *_obj, void (OBJ::*_cb)(T1))
{
	if (_addr != _m->getAddress())
		return;
	char tag[3];
	tag[0] = oscType<T1>::tag;
	tag[1] = '\0';
	if (_m->getTag() != tag)
		return;
	T1 p1; _m->get(p1);
	(_obj->*_cb)(p1);
}

template<typename OBJ, typename T1, typename T2>
void chain(MessagePtr _m, const string &_addr, OBJ *_obj, void (OBJ::*_cb)(T1, T2))
{
	if (_addr != _m->getAddress())
		return;
	char tag[3];
	tag[0] = oscType<T1>::tag;
	tag[1] = oscType<T2>::tag;
	tag[2] = '\0';
	if (_m->getTag() != tag)
		return;
	T1 p1; _m->get(p1);
	T2 p2; _m->get(p2);
	(_obj->*_cb)(p1, p2);
}

template<typename OBJ, typename T1, typename T2, typename T3>
void chain(MessagePtr _m, const string &_addr, OBJ *_obj, void (OBJ::*_cb)(T1, T2, T3))
{
	if (_addr != _m->getAddress())
		return;
	char tag[4];
	tag[0] = oscType<T1>::tag;
	tag[1] = oscType<T2>::tag;
	tag[2] = oscType<T3>::tag;
	tag[3] = '\0';
	if (_m->getTag() != tag)
		return;
	T1 p1; _m->get(p1);
	T2 p2; _m->get(p2);
	T3 p3; _m->get(p3);
	(_obj->*_cb)(p1, p2, p3);
}

template<typename OBJ, typename T1, typename T2, typename T3, typename T4>
void chain(MessagePtr _m, const string &_addr, OBJ *_obj, void (OBJ::*_cb)(T1, T2, T3, T4))
{
	if (_addr != _m->getAddress())
		return;
	char tag[5];
	tag[0] = oscType<T1>::tag;
	tag[1] = oscType<T2>::tag;
	tag[2] = oscType<T3>::tag;
	tag[3] = oscType<T4>::tag;
	tag[4] = '\0';
	if (_m->getTag() != tag)
		return;
	T1 p1; _m->get(p1);
	T2 p2; _m->get(p2);
	T3 p3; _m->get(p3);
	T4 p4; _m->get(p4);
	(_obj->*_cb)(p1, p2, p3, p4);
}

template<typename OBJ, typename T1, typename T2, typename T3, typename T4, typename T5>
void chain(MessagePtr _m, const string &_addr, OBJ *_obj, void (OBJ::*_cb)(T1, T2, T3, T4, T5))
{
	if (_addr != _m->getAddress())
		return;
	char tag[6];
	tag[0] = oscType<T1>::tag;
	tag[1] = oscType<T2>::tag;
	tag[2] = oscType<T3>::tag;
	tag[3] = oscType<T4>::tag;
	tag[4] = oscType<T5>::tag;
	tag[5] = '\0';
	if (_m->getTag() != tag)
		return;
	T1 p1; _m->get(p1);
	T2 p2; _m->get(p2);
	T3 p3; _m->get(p3);
	T4 p4; _m->get(p4);
	T5 p5; _m->get(p5);
	(_obj->*_cb)(p1, p2, p3, p4, p5);
}

inline void make(MessagePtr _m, const string &_addr)
{
	_m->begin(_addr);
	_m->end();
}

template<typename T1>
static void make(MessagePtr _m, const string &_addr, T1 _p1)
{
	_m->begin(_addr);
	_m->write(_p1);
	_m->end();
}

template<typename T1, typename T2>
static void make(MessagePtr _m, const string &_addr, T1 _p1, T2 _p2)
{
	_m->begin(_addr);
	_m->write(_p1);
	_m->write(_p2);
	_m->end();
}

template<typename T1, typename T2, typename T3>
static void make(MessagePtr _m, const string &_addr, T1 _p1, T2 _p2, T3 _p3)
{
	_m->begin(_addr);
	_m->write(_p1);
	_m->write(_p2);
	_m->write(_p3);
	_m->end();
}

template<typename T1, typename T2, typename T3, typename T4>
static void make(MessagePtr _m, const string &_addr, T1 _p1, T2 _p2, T3 _p3, T4 _p4)
{
	_m->begin(_addr);
	_m->write(_p1);
	_m->write(_p2);
	_m->write(_p3);
	_m->write(_p4);
	_m->end();
}

template<typename T1, typename T2, typename T3, typename T4, typename T5>
static void make(MessagePtr _m, const string &_addr, T1 _p1, T2 _p2, T3 _p3, T4 _p4, T5 _p5)
{
	_m->begin(_addr);
	_m->write(_p1);
	_m->write(_p2);
	_m->write(_p3);
	_m->write(_p4);
	_m->write(_p5);
	_m->end();
}

inline MessagePtr make(const string &_addr)
{
	MessagePtr m(new Message());
	m->begin(_addr);
	m->end();
	return m;
}

template<typename T1>
static MessagePtr make(const string &_addr, T1 _p1)
{
	MessagePtr m(new Message());
	m->begin(_addr);
	m->write(_p1);
	m->end();
	return m;
}

template<typename T1, typename T2>
static MessagePtr make(const string &_addr, T1 _p1, T2 _p2)
{
	MessagePtr m(new Message());
	m->begin(_addr);
	m->write(_p1);
	m->write(_p2);
	m->end();
	return m;
}

template<typename T1, typename T2, typename T3>
static MessagePtr make(const string &_addr, T1 _p1, T2 _p2, T3 _p3)
{
	MessagePtr m(new Message());
	m->begin(_addr);
	m->write(_p1);
	m->write(_p2);
	m->write(_p3);
	m->end();
	return m;
}

template<typename T1, typename T2, typename T3, typename T4>
static MessagePtr make(const string &_addr, T1 _p1, T2 _p2, T3 _p3, T4 _p4)
{
	MessagePtr m(new Message());
	m->begin(_addr);
	m->write(_p1);
	m->write(_p2);
	m->write(_p3);
	m->write(_p4);
	m->end();
	return m;
}

template<typename T1, typename T2, typename T3, typename T4, typename T5>
static MessagePtr make(const string &_addr, T1 _p1, T2 _p2, T3 _p3, T4 _p4, T5 _p5)
{
	MessagePtr m(new Message());
	m->begin(_addr);
	m->write(_p1);
	m->write(_p2);
	m->write(_p3);
	m->write(_p4);
	m->write(_p5);
	m->end();
	return m;
}

} // namespace osc

#endif // DISPATCHER_H_INDCLUDED
