/*
 *  This file is part of libosc++.
 *
 *  libosc++ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  libosc++ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesse General Public License
 *  along with libosc++.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright 2009 Cristian Maglie
 *
 *  Cristian Maglie <liboscpp@bug.st>
 *  http://bug.st/libosc++
 */

#ifndef OSCOSTREAM_H_INCLUDED
#define OSCOSTREAM_H_INCLUDED

#include <libosc++/exception.h>
#include <libosc++/message.h>
#include <libosc++/bundle.h>

#include <string>
#include <boost/shared_ptr.hpp>

using namespace std;
using namespace boost;

namespace osc {

struct beginbundle {
	explicit beginbundle(uint64_t _t = 1) : time(_t) { }
	uint64_t time;
};

struct beginmessage {
	explicit beginmessage(const string &_a) : addr(_a) { }
	string addr;
};

struct close { };

class oscostream {
public:
	oscostream();
	~oscostream();

	void clear();

	oscostream& operator<<(const beginbundle &_v);
	oscostream& operator<<(const beginmessage &_v);
	oscostream& operator<<(const close &_v);

	oscostream& operator<<(const int32_t _v);
	oscostream& operator<<(const bool _v);
	oscostream& operator<<(const nil &_v);
	oscostream& operator<<(const inf &_v);
	oscostream& operator<<(const char *_v);
	oscostream& operator<<(const float _v);
	oscostream& operator<<(const double _v);
	oscostream& operator<<(const char _v);

	bool ready() const;
	unsigned int size() const;
	void output(char *_buff) const;
	void output(string &_buff) const;

private:
	bool messageOpened;
	MessagePtr message;

	bool bundleOpened;
	BundlePtr bundle;

	bool childrenBundleOpened;
	shared_ptr<oscostream> children;

	string data;
};

class MessageOpenedException : public Exception {
public:
	MessageOpenedException() :
		Exception("Message opened.") { }
};

class MessageNotOpenedException : public Exception {
public:
	MessageNotOpenedException() :
		Exception("Message not opened.") { }
};

class BundleOpenedException : public Exception {
public:
	BundleOpenedException() :
		Exception("Bundle opened.") { }
};

class BundleNotOpenedException : public Exception {
public:
	BundleNotOpenedException() :
		Exception("Bundle not opened.") { }
};

} // namespace osc

#endif // OSCOSTREAM_H_INCLUDED
