/*
 *  This file is part of libosc++.
 *
 *  libosc++ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  libosc++ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesse General Public License
 *  along with libosc++.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright 2009 Cristian Maglie
 *
 *  Cristian Maglie <liboscpp@bug.st>
 *  http://bug.st/libosc++
 */

#ifndef OSCTYPES_H_INCLUDED
#define OSCTYPES_H_INCLUDED

#include <string>
#include <stdint.h>

using namespace std;

namespace osc {

struct nil { }; // OSC Null type

struct inf { }; // OSC Infinitum type

enum TypeTagValues {
	TRUE_TAG         = 'T',
	FALSE_TAG        = 'F',
	NIL_TAG          = 'N',
	INF_TAG          = 'I',
	INT32_TAG        = 'i',
	FLOAT32_TAG      = 'f',
	FLOAT64_TAG      = 'd',
	CHAR_TAG         = 'c',
	RGBA_COLOR_TAG   = 'r',
	MIDI_MESSAGE_TAG = 'm',
	INT64_TAG        = 'h',
	TIME_TAG_TAG     = 't',
	STRING_TAG       = 's',
	SYMBOL_TAG       = 'S',
	BLOB_TAG         = 'b'
};

template<typename T>
struct oscType {
	static const char tag = '\0';
};

template<> struct oscType<int32_t> {
	static const char tag = INT32_TAG;
};

template<> struct oscType<int64_t> {
	static const char tag = INT64_TAG;
};

template<> struct oscType<float> {
	static const char tag = FLOAT32_TAG;
};

template<> struct oscType<double> {
	static const char tag = FLOAT64_TAG;
};

template<> struct oscType<const char *> {
	static const char tag = STRING_TAG;
};

template<> struct oscType<char *> {
	static const char tag = STRING_TAG;
};

template<> struct oscType<string> {
	static const char tag = STRING_TAG;
};

template<> struct oscType<char> {
	static const char tag = CHAR_TAG;
};

template<> struct oscType<nil> {
	static const char tag = NIL_TAG;
};

template<> struct oscType<inf> {
	static const char tag = INF_TAG;
};

template<typename T> inline char typeToTag(T _v) {
	return oscType<T>::tag;
}

inline char typeToTag(bool _v) {
	return _v ? TRUE_TAG : FALSE_TAG;
}

}; // namespace osc

#endif // OSCTYPES_H_INCLUDED
