/*
 *  This file is part of libosc++.
 *
 *  libosc++ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  libosc++ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesse General Public License
 *  along with libosc++.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright 2009 Cristian Maglie
 *
 *  Cristian Maglie <liboscpp@bug.st>
 *  http://bug.st/libosc++
 */

#ifndef UDP_H_INCLUDED
#define UDP_H_INCLUDED

#include <boost/function.hpp>
#include <boost/asio.hpp>
#include <libosc++/message.h>

using namespace boost;
using namespace boost::asio;
using boost::asio::ip::udp;

namespace osc
{

class UDPTransport
{
public:
	UDPTransport();
	~UDPTransport();

	void setRecvUDPEndpoint(string _addr, string _port);
	void setRecvCB(function<void (MessagePtr)> _cb);
	void startRecvThread();

	void setSendUDPEndpoint(string _addr, string _port);
	void send(osc::MessagePtr);

private:
	function<void (MessagePtr)> recvCB;

	io_service io;
	udp::endpoint recvEndpoint, sendEndpoint;
	udp::endpoint resolve(string _addr, string _port);

	udp::socket socket;

	void run();
};

typedef shared_ptr<UDPTransport> UDPTransportPtr;

} // namespace osc

#endif // UDP_H_INCLUDED
