
#include <libosc++/bundle.h>
#include <libosc++/message.h>
#include <libosc++/dispatcher.h>
#include <libosc++/oscostream.h>

#include <boost/lexical_cast.hpp>
#include <iostream>
#include <iomanip>
#include <sstream>

using namespace std;
using namespace osc;

char buffer[4000];

void dump(char *buff, int size)
{
	int off = 0;

	ostringstream asciioss, hexoss;

	while (off<size) {
		if (off % 16 == 0) {
			hexoss << "0x" << hex << setfill('0') << setw(4) << off << "  ";
		}

		char cur = buff[off];
		hexoss << hex << setfill('0') << setw(2);
		hexoss << ((int)(cur) & 0xFF) << " ";

		if (cur>31 && cur<128)
			asciioss << cur;
		else
			asciioss << ".";

		if (off % 4 == 3) {
			hexoss << "  ";
			asciioss << " ";
		}

		if (off % 16 == 15) {
			cout << hexoss.str() << "   " << asciioss.str() << endl;
			hexoss.str(""); asciioss.str("");
		}
		off++;
	}
	cout << hexoss.str() << "   " << asciioss.str() << endl;
}

string polymorphCheckResult;

void cb0() {
	cout << "Callback0!" << endl;
	polymorphCheckResult += "cb0()";
}

void cb1(int32_t _p) {
	cout << "Callback1(" << _p << ")" << endl;
	polymorphCheckResult += "cb1(";
	polymorphCheckResult += lexical_cast<string>(_p);
	polymorphCheckResult += ")";
}

void poly0() {
	cout << "Polymorphic0" << endl;
	polymorphCheckResult += "poly0()";
}

void poly1(int32_t _p) {
	cout << "Polymorphic1(" << _p << ")" << endl;
	polymorphCheckResult += "poly1(";
	polymorphCheckResult += lexical_cast<string>(_p);
	polymorphCheckResult += ")";
}

void dispatch(MessagePtr _m) {
	chain(_m, "/cb0", cb0);
	chain(_m, "/cb1", cb1);
	chain(_m, "/polymorphic", poly0);
	chain(_m, "/polymorphic", poly1);
	polymorphCheckResult += "/";
}

bool checkCallbacks() {
	polymorphCheckResult = "";

	MessagePtr m_cb0 = make("/cb0");
	MessagePtr m_cb1 = make("/cb1", 666);
	MessagePtr m_cb1_no = make("/cb1", 666, 777);
	dispatch(m_cb0);
	dispatch(m_cb1);
	dispatch(m_cb1_no);

	MessagePtr m_poly0 = make("/polymorphic");
	MessagePtr m_poly1 = make("/polymorphic", 999);
	dispatch(m_poly0);
	dispatch(m_poly1);

	cout << polymorphCheckResult << endl;
	return true;
}

bool checkTypeToTagsFunctions()
{
	string tagsTest = "";
	tagsTest += oscType<int32_t>::tag;
	tagsTest += oscType<int64_t>::tag;
	tagsTest += oscType<float>::tag;
	tagsTest += oscType<double>::tag;
	tagsTest += oscType<string>::tag;
	tagsTest += oscType<char *>::tag;
	tagsTest += oscType<const char *>::tag;

	cout << "Tag Check result: " << tagsTest << endl;
	return true;
}

int main()
{
	Message msg;
	msg.begin("/pippo");
	msg.writeInt32(0x1234ABCD);
	msg.writeFloat32(-8346975.0);
	msg.writeFloat64(-1.9111033814895987e+53);
	msg.writeString("cicciao");
	msg.writeInf();
	msg.writeBool(true);
	msg.writeBool(false);
	msg.end();

	Bundle bnd;
	bnd.begin();
	bnd.writeMessage(msg);
	bnd.writeMessage(msg);
	bnd.end();

	Bundle bnd2;
	bnd2.begin(0x123456789ABCDEFLL);
	bnd2.writeMessage(msg);
	bnd2.writeBundle(bnd);
	bnd2.end();
	bnd2.output(buffer);

	dump(buffer, bnd2.size());

	oscostream str;
	str << beginbundle(0x123456789ABCDEFLL)
			<< beginmessage("/pippo")
				<< 0x1234ABCD
				<< float(-8346975.0)
				<< double(-1.9111033814895987e+53)
				<< "cicciao"
				<< inf()
				<< true
				<< false
			<< osc::close()
			<< beginbundle()
				<< beginmessage("/pippo")
					<< 0x12345678
					<< 0x12345678
					<< "cicciao"
					<< inf()
					<< true
					<< false
				<< osc::close()
				<< beginmessage("/pippo")
					<< 0x12345678
					<< 0x12345678
					<< "cicciao"
					<< inf()
					<< true
					<< false
				<< osc::close()
			<< osc::close()
		<< osc::close();
	str.output(buffer);
	dump(buffer, str.size());

	checkTypeToTagsFunctions();

	Message test;
	test.begin("/calippofizz");
	test.writeInt32(10);
	test.writeFloat32(20.5);
	test.writeString("Giuseppina tornatore");
	test.writeFloat64(40.5);
	test.end();

	Message recv;
	test.output(buffer);
	dump(buffer, test.size());
	recv.input(buffer, test.size());
	cout << recv.getAddress() << endl << recv.getTag() << endl;
	cout << recv.getInt32() << endl;
	cout << recv.getFloat32() << endl;
	cout << recv.getString() << endl;
	cout << recv.getFloat64() << endl;

	checkCallbacks();

	MessagePtr m = osc::make("/din/0", 1);
	m->output(buffer);
	dump(buffer, m->size());

	m = osc::make("/ain/0", 1023);
	m->output(buffer);
	dump(buffer, m->size());

	return 0;
}

